
import types


DUMMY_PERMISSION = 'dummy_permission'
ANOTHER_PERMISSION = 'another_permission'
YET_ANOTHER_PERMISSION = 'yet_another_permission'

DUMMY_ACCOUNT = types.SimpleNamespace(role=DUMMY_PERMISSION)
ANOTHER_ACCOUNT = types.SimpleNamespace(role=ANOTHER_PERMISSION)
